@@ -16,8 +16,10 @@
 import copy
 
 import sahara.exceptions as ex
+from sahara.i18n import _
 import sahara.plugins.base as plugin_base
 import sahara.service.api as api
+from sahara.service import ops
 import sahara.service.validations.base as b
 import sahara.service.validations.cluster_templates as cl_t
 
@@ -68,22 +70,43 @@ CLUSTER_SCALING_SCHEMA = {
 
 def check_cluster_scaling(data, cluster_id, **kwargs):
     cluster = api.get_cluster(id=cluster_id)
+
+    cluster_engine = cluster.sahara_info.get(
+        'infrastructure_engine') if cluster.sahara_info else None
+
+    if (not cluster_engine and
+            not ops.get_engine_type_and_version().startswith('direct')):
+        raise ex.InvalidException(
+            _("Cluster created before Juno release "
+              "can't be scaled with %(engine)s engine") %
+            {"engine": ops.get_engine_type_and_version()})
+
+    if (cluster.sahara_info and
+            cluster_engine != ops.get_engine_type_and_version()):
+        raise ex.InvalidException(
+            _("Cluster created with %(old_engine)s infrastructure engine "
+              "can't be scaled with %(new_engine)s engine") %
+            {"old_engine": cluster.sahara_info.get('infrastructure_engine'),
+             "new_engine": ops.get_engine_type_and_version()})
+
     if not (plugin_base.PLUGINS.is_plugin_implements(cluster.plugin_name,
                                                      'scale_cluster') and (
             plugin_base.PLUGINS.is_plugin_implements(cluster.plugin_name,
                                                      'decommission_nodes'))):
         raise ex.InvalidException(
-            "Requested plugin '%s' doesn't support cluster scaling feature"
+            _("Requested plugin '%s' doesn't support cluster scaling feature")
             % cluster.plugin_name)
 
     if cluster.status != 'Active':
-        raise ex.InvalidException("Cluster cannot be scaled not in 'Active' "
-                                  "status. Cluster status: " + cluster.status)
+        raise ex.InvalidException(
+            _("Cluster cannot be scaled not in 'Active' status. "
+              "Cluster status: %s") % cluster.status)
 
     if data.get("resize_node_groups"):
         b.check_resize(cluster, data['resize_node_groups'])
 
     if data.get("add_node_groups"):
         b.check_add_node_groups(cluster, data['add_node_groups'])
-
         b.check_network_config(data['add_node_groups'])
+        b.check_cluster_hostnames_lengths(cluster.name,
+                                          data['add_node_groups'])
