@@ -15,10 +15,11 @@
 
 import copy
 
-import unittest2
+import testtools
 
 from sahara.conductor import resource as r
 from sahara.swift import swift_helper
+from sahara.utils import edp
 from sahara.utils import types
 
 
@@ -67,13 +68,13 @@ SAMPLE_JOB_BINARY_DICT = {
     "name": "bob",
     "tenant_id": "6b859fb8d1f44e8eafdfb91f21309b5f",
     "updated_at": "null",
-    "url": "swift-internal://bob.sahara/job"
+    "url": "swift://bob.sahara/job"
 }
 
 SAMPLE_JOB_BINARY_DICT2 = copy.copy(SAMPLE_JOB_BINARY_DICT)
 SAMPLE_JOB_BINARY_DICT2["name"] = "bill"
 SAMPLE_JOB_BINARY_DICT2["id"] = "c0caf119-1111-2222-a46e-0f28ebd23b5c"
-SAMPLE_JOB_BINARY_DICT2["url"] = "swift-internal://bill.sahara/job"
+SAMPLE_JOB_BINARY_DICT2["url"] = "swift://bill.sahara/job"
 
 SAMPLE_JOB_DICT = {
     "tenant_id": "test_tenant",
@@ -103,7 +104,7 @@ SAMPLE_JOB_EXECUTION = {
     "info": {
         "actions": [{"conf": "some stuff"},
                     {"conf": "more stuff"}],
-        "status": "Pending"
+        "status": edp.JOB_STATUS_PENDING
     },
     "input_id": "b5ddde55-594e-428f-9040-028be81eb3c2",
     "job_configs": {
@@ -115,6 +116,14 @@ SAMPLE_JOB_EXECUTION = {
             swift_helper.HADOOP_SWIFT_PASSWORD: "openstack",
             swift_helper.HADOOP_SWIFT_USERNAME: "admin",
             "myfavoriteconfig": 1
+        },
+        "proxy_configs": {
+            "proxy_username": "admin",
+            "proxy_password": "openstack"
+        },
+        "trusts": {
+            "input_id": "9c528755099149b8b7166f3d0fa3bf10",
+            "output_id": "3f2bde9d43ec440381dc9f736481e2b0"
         }
     },
     "job_id": "d0f3e397-7bef-42f9-a4db-e5a96059246e",
@@ -123,10 +132,7 @@ SAMPLE_JOB_EXECUTION = {
 }
 
 
-class TestResource(unittest2.TestCase):
-    def setUp(self):
-        self.maxDiff = None
-
+class TestResource(testtools.TestCase):
     def test_resource_creation(self):
         res = r.Resource(SAMPLE_DICT)
 
@@ -139,13 +145,13 @@ class TestResource(unittest2.TestCase):
     def test_resource_immutability(self):
         res = r.Resource(SAMPLE_DICT)
 
-        with self.assertRaises(types.FrozenClassError):
+        with testtools.ExpectedException(types.FrozenClassError):
             res.first.append(123)
 
-        with self.assertRaises(types.FrozenClassError):
+        with testtools.ExpectedException(types.FrozenClassError):
             res.first = 123
 
-        with self.assertRaises(types.FrozenClassError):
+        with testtools.ExpectedException(types.FrozenClassError):
             res.second.a = 123
 
     def test_nested_lists(self):
@@ -214,6 +220,14 @@ class TestResource(unittest2.TestCase):
                       job_exec['job_configs']['configs'])
         for a in job_exec['info']['actions']:
             self.assertIn('conf', a)
+        self.assertIn('trusts', job_exec['job_configs'])
+        self.assertIn('input_id', job_exec['job_configs']['trusts'])
+        self.assertIn('output_id', job_exec['job_configs']['trusts'])
+        self.assertIn('proxy_configs', job_exec['job_configs'])
+        self.assertIn('proxy_username',
+                      job_exec['job_configs']['proxy_configs'])
+        self.assertIn('proxy_password',
+                      job_exec['job_configs']['proxy_configs'])
 
         wrapped_dict = job_exec.to_wrapped_dict()['job_execution']
         self.assertNotIn('extra', wrapped_dict)
@@ -226,3 +240,6 @@ class TestResource(unittest2.TestCase):
 
         for a in wrapped_dict['info']['actions']:
             self.assertNotIn('conf', a)
+
+        self.assertNotIn('trusts', wrapped_dict['job_configs'])
+        self.assertNotIn('proxy_configs', wrapped_dict['job_configs'])
