@@ -210,3 +210,16 @@ class NetcatSocket:
     def reset(self):
         self._terminate()
         self._create_process()
+
+
+def get_private_network_cidrs(cluster):
+    neutron_client = client()
+    private_net = neutron_client.show_network(
+        cluster.neutron_management_network)
+
+    cidrs = []
+    for subnet_id in private_net['network']['subnets']:
+        subnet = neutron_client.show_subnet(subnet_id)
+        cidrs.append(subnet['subnet']['cidr'])
+
+    return cidrs
